@@ -60,7 +60,11 @@ vcov.gel <- function(object, lambda = FALSE, ...)
 
 print.gel <- function(x, digits = 5, ...)
 	{
-	cat("Type de GEL: ", x$type, "\n\n")
+	if (is.null(x$CGEL))
+		cat("Type de GEL: ", x$type, "\n\n")
+	else
+		cat("CGEL of type: ", x$type, " (alpha = ", x$CGEL, ")\n\n")
+
 	cat("Coefficients:\n")
 	print.default(format(coef(x), digits = digits),
                       print.gap = 2, quote = FALSE)
@@ -68,6 +72,9 @@ print.gel <- function(x, digits = 5, ...)
 	cat("Lambdas:\n")
 	print.default(format(coef(x, lambda = TRUE), digits = digits),
                       print.gap = 2, quote = FALSE)
+	cat("\n")
+	cat("Convergence code for the coefficients: ", x$conv_par,"\n")
+	cat("Convergence code for Lambda: ", x$conv_lambda$convergence,"\n")
 	invisible(x)
 	}
 
@@ -75,7 +82,10 @@ print.summary.gel <- function(x, digits = 5, ...)
 	{
 	cat("\nCall:\n")
 	cat(paste(deparse(x$call), sep = "\n", collapse = "\n"), "\n\n", sep = "")
-	cat("\nType of GEL: ", x$type, "\n\n")
+	if (is.null(x$CGEL))
+		cat("Type de GEL: ", x$type, "\n\n")
+	else
+		cat("CGEL of type: ", x$type, " (alpha = ", x$CGEL, ")\n\n")
 	cat("Kernel: ", x$kernel, "\n\n")
 	cat("Coefficients:\n")
 	print.default(format(x$coefficients, digits = digits),
@@ -90,7 +100,7 @@ print.summary.gel <- function(x, digits = 5, ...)
                       print.gap = 2, quote = FALSE)
 
 	cat("\nConvergence code for the coefficients: ", x$conv_par, "\n")
-	cat("\nConvergence code for the lambdas: ", x$conv_lambda, "\n")
+	cat("\nConvergence code for the lambdas: ", x$conv_lambda$convergence, "\n")
 	
 	invisible(x)
 	}
@@ -130,9 +140,8 @@ summary.gel <- function(object, ...)
 	ans$conv_pt <- z$conv_pt
 	ans$conv_moment <- cbind(z$conv_moment)
 	ans$conv_lambda <- z$conv_lambda
-	names(ans$conv_par) <- "Convergence_code_theta"
+	ans$CGEL <- z$CGEL
 	names(ans$conv_pt) <- "Sum_of_pt"
-	names(ans$conv_lambda) <- "Convergence_code_for_lambda"
 	dimnames(ans$conv_moment) <- list(names(z$gt), "Sample_moment_with_pt")
 	class(ans) <- "summary.gel"
 	ans	
